@@ -2,8 +2,8 @@
 # run qmake for all pro files in project tree
 PROFILES=`find . -name "*.pro"`
 for i in $PROFILES; do
-    pushd .
+    pushd . > /dev/null
     cd `dirname $i`
     qmake
-    popd
+    popd > /dev/null
 done
