@@ -1,35 +1,23 @@
 // Filename: mainwindow.cpp
 
 #include <QDebug>
-#include <QMainWindow>
 #include <QMessageBox>
 
 #include "mainwindow.h"
 #include "mapwidget.h"
 #include "mapdialog.h"
 #include "iconloader.h"  
+#include "cachewindow.h"
+
 #include <QIcon>
+#include <QSettings>
 
-#ifdef Q_WS_MAEMO_5
-#include <QMaemo5InformationBox>
-#include <QtGui/QX11Info>
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-
-void grabZoomKeys(QWidget *window) {
-  if(!window->winId()) return;
-  
-  unsigned long val = 1;
-  Atom atom = XInternAtom(QX11Info::display(), "_HILDON_ZOOM_KEY_ATOM", False);
-  if(!atom) return;
-  
-  XChangeProperty (QX11Info::display(), window->winId(), atom, 
-		   XA_INTEGER, 32, PropModeReplace,
-		   reinterpret_cast<unsigned char *>(&val),  1);
-}
-#else
+#ifndef Q_WS_MAEMO_5
 #include <QStatusBar>
+#include <QProgressBar>
 #include <QVBoxLayout>
+#else
+#include <QMaemo5InformationBox>
 #endif
 
 void MainWindow::createMenu() {
@@ -51,6 +39,10 @@ void MainWindow::createMenu() {
   menu->addAction(action);
   menuBar()->addMenu(menu);
 
+  menu = new QMenu(tr("&Plugins"), this);
+  m_cacheProvider->createMenu(menu);
+  menuBar()->addMenu(menu);
+
   menu = new QMenu(tr("&Help"), this);
   action = new QAction(tr("&About"), this);
   connect(action, SIGNAL(triggered()), this, SLOT(about()));
@@ -76,9 +68,10 @@ void MainWindow::about() {
 
 
 MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) : 
-  QMainWindow(parent), m_locationProvider(locationProvider) {
+  CustomWindow("MainWindow", parent), m_locationProvider(locationProvider) {
 
-  qDebug() << __FUNCTION__ << "Cache provider " << m_cacheProvider.name();
+  qDebug() << __FUNCTION__;
+  m_cacheProvider = new CacheProvider();
 
   setWindowTitle(tr("CacheMe"));
 
@@ -86,11 +79,6 @@ MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) :
   QPixmap *icon = iconLoader.load(APPNAME);
   if(icon) setWindowIcon(QIcon(*icon));
 
-#ifdef Q_WS_MAEMO_5
-  setAttribute(Qt::WA_Maemo5AutoOrientation, true);
-  grabZoomKeys(this);
-#endif
-
   createMenu();
 
   // search for service providers
@@ -116,40 +104,52 @@ MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) :
   QObject::connect(this->m_locationProvider, SIGNAL(positionUpdated(const QGeoPositionInfo &)), 
 		   this->m_mapWidget, SLOT(positionUpdated(const QGeoPositionInfo &)));
 
-  QObject::connect(&this->m_cacheProvider, SIGNAL(replyOverview(const CacheList &)), 
+  QObject::connect(this->m_cacheProvider, SIGNAL(replyOverview(const CacheList &)), 
 		   this, SLOT(updateCaches(const CacheList &)));
 
-  QObject::connect(&this->m_cacheProvider, SIGNAL(replyInfo(const Cache &)), 
+  QObject::connect(this->m_cacheProvider, SIGNAL(replyInfo(const Cache &)), 
 		   this->m_mapWidget, SLOT(showBubble(const Cache &)));
 
-  QObject::connect(&this->m_cacheProvider, SIGNAL(replyError(const QString &)), 
+  QObject::connect(this->m_cacheProvider, SIGNAL(replyDetail(const Cache &)), 
+		   this, SLOT(showDetail(const Cache &)));
+
+  QObject::connect(this->m_cacheProvider, SIGNAL(replyError(const QString &)), 
 		   this, SLOT(displayError(const QString &)));
 
+  QObject::connect(this->m_cacheProvider, SIGNAL(settingsChanged()),
+		   this, SLOT(reloadCaches()));
+
+  QObject::connect(this->m_cacheProvider, SIGNAL(busy(bool)),
+		   this, SLOT(setBusy(bool)));
+
   QObject::connect(this->m_mapWidget, SIGNAL(mapChanged()),
 		   this, SLOT(reloadCaches()));
 
   QObject::connect(this->m_mapWidget, SIGNAL(showMessage(const QString &)),
 		   this, SLOT(statusMessage(const QString &)));
 
-  QObject::connect(this, SIGNAL(increasePressed()),
+  QObject::connect(this, SIGNAL(zoomInPressed()),
 		   this->m_mapWidget, SLOT(zoomIn()));
 
-  QObject::connect(this, SIGNAL(decreasePressed()),
+  QObject::connect(this, SIGNAL(zoomOutPressed()),
 		   this->m_mapWidget, SLOT(zoomOut()));
 
-  // connect to map to handle clicks on cache icons
+  // connect to map to handle clicks on cache icons ...
   QObject::connect(this->m_mapWidget, SIGNAL(cacheClicked(const QString &)),
 		   this, SLOT(selectCache(const QString &)));
 
+  // ... and the detail button
+  QObject::connect(this->m_mapWidget, SIGNAL(detailClicked(const QString &)),
+		   this, SLOT(detailCache(const QString &)));
+
+  // build a graphics scene based on the mapwidget
   QGraphicsScene *scene = new QGraphicsScene(this);
   m_view = new QGraphicsView( scene );
   m_view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   m_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   m_view->setVisible(true);
   m_view->setInteractive(true);
-
   scene->addItem(m_mapWidget);
-
   setCentralWidget(m_view);
 
 #ifndef Q_WS_MAEMO_5
@@ -157,21 +157,12 @@ MainWindow::MainWindow(LocationProvider *locationProvider, QWidget *parent) :
   setStatusBar(new QStatusBar(this));
 #endif
 
-  settings.beginGroup("MainWindow");
-  resize(settings.value("size", QSize(640, 480)).toSize());
-  move(settings.value("pos", QPoint(200, 200)).toPoint());
-  settings.endGroup();
+  m_cacheProvider->start();
 }
 
 MainWindow::~MainWindow() {
   qDebug() << __FUNCTION__;
 
-  QSettings settings;
-  settings.beginGroup("MainWindow");
-  settings.setValue("size", size());
-  settings.setValue("pos", pos());
-  settings.endGroup();
-  
   delete m_serviceProvider;
 }
 
@@ -180,64 +171,77 @@ void MainWindow::resizeEvent(QResizeEvent*) {
   m_mapWidget->resize(m_view->size());
 }
 
-void MainWindow::keyPressEvent(QKeyEvent* event) {
-  // check if device is in portrait mode
-  bool portrait = size().height() > size().width();
-
-  switch (event->key()) {
-  case Qt::Key_F7:
-    if(portrait) emit decreasePressed();
-    else         emit increasePressed();
-    event->accept();
-    break;
-    
-  case Qt::Key_F8:
-    if(portrait) emit increasePressed();
-    else         emit decreasePressed();
-    event->accept();
-    break;
-  }
-  QWidget::keyPressEvent(event);
+void MainWindow::selectCache(const QString &name) {
+  // ask CacheProvider for info about this cache
+  this->m_cacheProvider->requestInfo(name);
 }
 
-void MainWindow::selectCache(const QString &name) {
+void MainWindow::detailCache(const QString &name) {
   // ask CacheProvider for details about this cache
-  this->m_cacheProvider.requestInfo(name);
+  this->m_cacheProvider->requestDetail(name);
 }
 
 void MainWindow::reloadCaches() {
   // request new caches for the maps new viewport
-  m_cacheProvider.requestOverview(this->m_mapWidget->viewport());
+  m_cacheProvider->requestOverview(this->m_mapWidget->viewport());
 }
 
 void MainWindow::updateCaches(const CacheList &cacheList) {
-#ifndef Q_WS_MAEMO_5
-  // check if it's actually one of the error messages that's still
-  // displayed and remove it in that case
-  if(statusBar()->property("error").toBool())  
-    statusBar()->clearMessage();
-#endif
-
   m_message = "";
   this->m_mapWidget->updateCaches(cacheList);
 }
 
-void MainWindow::statusMessage(const QString &message, int time) {
+void MainWindow::statusMessage(const QString &message) {
 #ifdef Q_WS_MAEMO_5
   QMaemo5InformationBox::information(this, message);
 #else
   statusBar()->setProperty("error", false);  
-  statusBar()->showMessage(message, time);
+  statusBar()->showMessage(message, 5000);
 #endif
 }
 
 void MainWindow::displayError(const QString &message) {
   if(m_message != message) {
-    statusMessage(message, 0);
+    statusMessage(message);
     m_message = message;
-
-#ifndef Q_WS_MAEMO_5
-    statusBar()->setProperty("error", true );
-#endif
   }
 }
+
+void MainWindow::showDetail(const Cache &cache) {
+  qDebug() << __FUNCTION__;
+
+  CacheWindow *cacheWindow = new CacheWindow(cache, this);
+  cacheWindow->show();
+}
+
+void MainWindow::setBusy(bool on) {
+  static int busyCount = 0;
+
+#ifdef Q_WS_MAEMO_5
+  if((!on && busyCount  > 0) || ( on && busyCount <= 0))
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, on);
+#else
+  static QProgressBar *progressBar = NULL;
+  if(on && busyCount <= 0) {
+    if(!progressBar) {
+      progressBar = new QProgressBar();
+      progressBar->setMinimum(0);
+      progressBar->setMaximum(0);
+      progressBar->setValue(25);
+      progressBar->setMaximumWidth(128);
+      progressBar->setMaximumHeight(16);
+      statusBar()->addPermanentWidget(progressBar);
+    } else {
+      statusBar()->addPermanentWidget(progressBar);
+      progressBar->reset();
+      progressBar->show();
+    }
+  } else if(!on && busyCount > 0) {
+    statusBar()->removeWidget(progressBar);
+  }
+#endif
+
+  if(on)                        busyCount++;
+  else if(!on && busyCount > 0) busyCount--;
+}
+
